from typing import Any

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.consts import ANY_VALUE


class OSSBucketAccessLogs(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure the OSS bucket has access logging enabled"
        id = "CKV_ALI_12"
        supported_resources = ("alicloud_oss_bucket",)
        categories = (CheckCategories.LOGGING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "logging"

    def get_expected_value(self) -> Any:
        return ANY_VALUE


check = OSSBucketAccessLogs()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class VnetSingleDNSServer(BaseResourceCheck):
    def __init__(self) -> None:
        """Using a single DNS server may indicate a single point of failure
        where the DNS IP address is not load balanced."""
        name = "Ensure that VNET has at least 2 connected DNS Endpoints"
        id = "CKV_AZURE_182"
        supported_resources = ("azurerm_virtual_network", "azurerm_virtual_network_dns_servers")
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        if "dns_servers" in conf and conf["dns_servers"] and isinstance(conf["dns_servers"], list):
            dns_servers = conf["dns_servers"][0]
            if dns_servers and len(dns_servers) == 1:
                self.evaluated_keys = ["dns_servers"]
                return CheckResult.FAILED
        return CheckResult.PASSED


check = VnetSingleDNSServer()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class WAFHasAnyRules(BaseResourceCheck):
    def __init__(self):
        name = "Ensure WAF has associated rules"
        id = "CKV_AWS_175"
        supported_resources = ['aws_waf_web_acl', 'aws_wafregional_web_acl', 'aws_wafv2_web_acl']
        categories = [CheckCategories.APPLICATION_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if "rules" in conf.keys() and conf["rules"] != [{}]:
            return CheckResult.PASSED
        if "rule" in conf.keys() and conf["rule"] != [{}]:
            return CheckResult.PASSED
        return CheckResult.FAILED


check = WAFHasAnyRules()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class APIManagementMinTLS12(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure API management uses at least TLS 1.2"
        id = "CKV_AZURE_173"
        supported_resources = ("azurerm_api_management",)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        if conf.get('security'):
            security = conf['security'][0]
            if 'enable_back_end_ssl30' in security:
                if security['enable_back_end_ssl30'][0]:
                    return CheckResult.FAILED
            if 'enable_backend_tls10' in security:
                if security['enable_backend_tls10'][0]:
                    return CheckResult.FAILED
            if 'enable_frontend_ssl30' in security:
                if security['enable_frontend_ssl30'][0]:
                    return CheckResult.FAILED
            if 'enable_frontend_tls10' in security:
                if security['enable_frontend_tls10'][0]:
                    return CheckResult.FAILED
            if 'enable_frontend_tls11' in security:
                if security['enable_frontend_tls11'][0]:
                    return CheckResult.FAILED
        return CheckResult.PASSED


check = APIManagementMinTLS12()



from __future__ import annotations

from typing import Dict, Any

from checkov.policies_3d.checks_infra.base_parser import Base3dPolicyCheckParser
from checkov.policies_3d.checks_infra.base_check import Base3dPolicyCheck


class Policy3dParser(Base3dPolicyCheckParser):
    def parse_raw_check(self, raw_check: Dict[str, Dict[str, Any]], **kwargs: Any) -> Base3dPolicyCheck:
        policy_definition = raw_check.get("definition", {})
        check = Base3dPolicyCheck()
        check.iac = policy_definition.get('iac', {})
        check.cve = policy_definition.get('cve', {})
        check.id = raw_check.get("metadata", {}).get("id", "")
        check.name = raw_check.get("metadata", {}).get("name", "")
        check.category = raw_check.get("metadata", {}).get("category", "")
        check.guideline = raw_check.get("metadata", {}).get("guideline")

        return check



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.azure.NSGRulePortAccessRestricted import INTERNET_ADDRESSES
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class NSGRuleUDPAccessRestricted(BaseResourceCheck):
    def __init__(self):
        name = "Ensure that UDP Services are restricted from the Internet "
        id = "CKV_AZURE_77"
        supported_resources = ['azurerm_network_security_group', 'azurerm_network_security_rule']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        rule_confs = [conf]
        evaluated_key_prefix = ''
        if 'security_rule' in conf:
            rule_confs = conf['security_rule']
            self.evaluated_keys = ['security_rule']
            evaluated_key_prefix = 'security_rule/'
        for rule_conf in rule_confs:
            if isinstance(rule_conf, dict):
                if 'protocol' in rule_conf and rule_conf['protocol'][0].lower() == 'udp' \
                        and 'direction' in rule_conf and rule_conf['direction'][0].lower() == 'inbound' \
                        and 'access' in rule_conf and rule_conf['access'][0].lower() == 'allow' \
                        and 'source_address_prefix' in rule_conf \
                        and rule_conf['source_address_prefix'][0].lower() in INTERNET_ADDRESSES:
                    evaluated_key_prefix = f'{evaluated_key_prefix}[{rule_confs.index(rule_conf)}]/' if \
                        evaluated_key_prefix else ''
                    self.evaluated_keys = [f'{evaluated_key_prefix}protocol',
                                           f'{evaluated_key_prefix}direction',
                                           f'{evaluated_key_prefix}access',
                                           f'{evaluated_key_prefix}source_address_prefix']
                    return CheckResult.FAILED
                return CheckResult.PASSED
            return CheckResult.UNKNOWN
        return CheckResult.PASSED


check = NSGRuleUDPAccessRestricted()



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.gcp.AbsGoogleBasicRoles import AbsGoogleBasicRoles


class GoogleProjectBasicRoles(AbsGoogleBasicRoles):
    def __init__(self) -> None:
        name = "Ensure basic roles are not used at project level."
        id = "CKV_GCP_117"
        supported_resources = ('google_project_iam_member', 'google_project_iam_binding')
        categories = (CheckCategories.IAM,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)


check = GoogleProjectBasicRoles()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class RDSEnhancedMonitorEnabled(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that enhanced monitoring is enabled for Amazon RDS instances"
        id = "CKV_AWS_118"
        supported_resources = ("aws_db_instance", "aws_rds_cluster_instance")
        categories = (CheckCategories.LOGGING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "monitoring_interval"

    def get_expected_values(self) -> list[Any]:
        return [1, 5, 10, 15, 30, 60]


check = RDSEnhancedMonitorEnabled()



# -*- coding: utf-8 -*-
"""
=================
stopit.signalstop
=================

Control the timeout of blocks or callables with a context manager or a
decorator. Based on the use of signal.SIGALRM
"""

import signal

from .utils import TimeoutException, BaseTimeout, base_timeoutable


class SignalTimeout(BaseTimeout):
    """Context manager for limiting in the time the execution of a block
    using signal.SIGALRM Unix signal.

    See :class:`stopit.utils.BaseTimeout` for more information
    """

    def __init__(self, seconds, swallow_exc=True):
        seconds = int(seconds)  # alarm delay for signal MUST be int
        super(SignalTimeout, self).__init__(seconds, swallow_exc)

    def handle_timeout(self, signum, frame):
        self.state = self.TIMED_OUT
        raise TimeoutException('Block exceeded maximum timeout '
                               'value (%d seconds).' % self.seconds)

    # Required overrides
    def setup_interrupt(self):
        signal.signal(signal.SIGALRM, self.handle_timeout)
        signal.alarm(self.seconds)

    def suppress_interrupt(self):
        signal.alarm(0)
        signal.signal(signal.SIGALRM, signal.SIG_DFL)


class signal_timeoutable(base_timeoutable):  # noqa: B903
    """A function or method decorator that raises a ``TimeoutException`` to
    decorated functions that should not last a certain amount of time.
    this one uses ``SignalTimeout`` context manager.

    See :class:`.utils.base_timoutable`` class for further comments.
    """
    def __init__(self):
        self.to_ctx_mgr = SignalTimeout



from dataclasses import dataclass
from typing import Optional


@dataclass
class ModuleParams:
    def __init__(self, root_dir: str, current_dir: str, source: str, source_version: Optional[str], dest_dir: str,
                 external_modules_folder_name: str, inner_module: Optional[str] = None):
        self.root_dir: str = root_dir
        self.current_dir: str = current_dir
        self.module_source: str = source
        self.version: Optional[str] = source_version
        self.dest_dir: str = dest_dir
        self.external_modules_folder_name: str = external_modules_folder_name
        self.inner_module: Optional[str] = inner_module

        self.token: Optional[str] = None
        self.username: Optional[str] = None
        self.vcs_base_url: Optional[str] = None
        self.module_source_prefix: Optional[str] = None
        self.REGISTRY_URL_PREFIX: Optional[str] = None
        self.best_version: Optional[str] = None
        self.module_version_url: Optional[str] = None



from dataclasses import dataclass


class SourceType:
    __slots__ = ("name", "upload_results", "report_contributor_metrics")

    def __init__(self, name: str, upload_results: bool, report_contributor_metrics: bool = False):
        self.name = name
        self.upload_results = upload_results
        self.report_contributor_metrics = report_contributor_metrics


@dataclass
class BCSourceType:
    VSCODE = 'vscode'
    JETBRAINS = 'jetbrains'
    CLI = 'cli'
    KUBERNETES_WORKLOADS = 'kubernetesWorkloads'
    GITHUB_ACTIONS = 'githubActions'
    CODEBUILD = 'codebuild'
    JENKINS = 'jenkins'
    ADMISSION_CONTROLLER = 'admissionController'
    CIRCLECI = 'circleci'
    DISABLED = 'disabled'  # use this as a placeholder for generic no-upload logic


SourceTypes = {
    BCSourceType.VSCODE: SourceType(BCSourceType.VSCODE, False),
    BCSourceType.JETBRAINS: SourceType(BCSourceType.JETBRAINS, False),
    BCSourceType.CLI: SourceType(BCSourceType.CLI, True),
    BCSourceType.KUBERNETES_WORKLOADS: SourceType(BCSourceType.KUBERNETES_WORKLOADS, True),
    BCSourceType.DISABLED: SourceType(BCSourceType.VSCODE, False),
    BCSourceType.GITHUB_ACTIONS: SourceType(BCSourceType.GITHUB_ACTIONS, True, report_contributor_metrics=True),
    BCSourceType.CODEBUILD: SourceType(BCSourceType.CODEBUILD, True, report_contributor_metrics=True),
    BCSourceType.JENKINS: SourceType(BCSourceType.JENKINS, True, report_contributor_metrics=True),
    BCSourceType.CIRCLECI: SourceType(BCSourceType.CIRCLECI, True, report_contributor_metrics=True),
    BCSourceType.ADMISSION_CONTROLLER: SourceType(BCSourceType.ADMISSION_CONTROLLER, False)
}


IDEsSourceTypes = {BCSourceType.VSCODE, BCSourceType.JETBRAINS}


def get_source_type(source: str) -> SourceType:
    # helper method to get the source type with a default - using dict.get is ugly; you have to do:
    # SourceTypes.get(xyz, SourceTypes[BCSourceType.Disabled])
    if source in SourceTypes:
        return SourceTypes[source]
    else:
        return SourceTypes[BCSourceType.CLI]



from __future__ import annotations

import itertools
from typing import Optional, List, Tuple, Dict, Any, TYPE_CHECKING

from checkov.common.graph.checks_infra.enums import Operators
from checkov.common.graph.checks_infra.solvers.base_solver import BaseSolver
from checkov.common.checks_infra.solvers.connections_solvers.complex_connection_solver import ComplexConnectionSolver
from checkov.common.graph.graph_builder.graph_components.attribute_names import CustomAttributes

if TYPE_CHECKING:
    from checkov.common.typing import LibraryGraph


class AndConnectionSolver(ComplexConnectionSolver):
    operator = Operators.AND  # noqa: CCE003  # a static attribute

    def __init__(self, solvers: Optional[List[BaseSolver]], operator: str) -> None:
        super().__init__(solvers, operator)

    def get_operation(self, graph_connector: LibraryGraph) -> \
            Tuple[List[Dict[str, Any]], List[Dict[str, Any]], List[Dict[str, Any]]]:
        if not self.vertices_under_resource_types:
            return [], [], []

        passed, failed, unknown = self.run_attribute_solvers(graph_connector)
        failed_or_unknown_ids = [f[CustomAttributes.ID] for f in itertools.chain(failed, unknown)]
        passed = [p for p in passed if p[CustomAttributes.ID] not in failed_or_unknown_ids]

        for connection_solver in self.get_sorted_connection_solvers():
            connection_solver.set_vertices(graph_connector, failed, unknown)
            passed_solver, failed_solver, unknown_solver = connection_solver.get_operation(graph_connector)
            passed.extend(passed_solver)
            failed.extend(failed_solver)
            unknown.extend(unknown_solver)
            failed_or_unknown_ids.extend(f[CustomAttributes.ID] for f in itertools.chain(failed_solver, unknown_solver))

        failed_ids = [f[CustomAttributes.ID] for f in failed]
        unknown_ids = [u[CustomAttributes.ID] for u in unknown]
        passed = [p for p in passed if p[CustomAttributes.ID] not in itertools.chain(failed_ids, unknown_ids)]
        unknown = [u for u in unknown if u[CustomAttributes.ID] not in failed_ids]
        return self.filter_results(passed, failed, unknown)

    def _get_operation(self, *args: Any, **kwargs: Any) -> None:
        pass
